// Copyright (C) 2004, Brian Enigma <enigma at netninja.com>
// This file is part of iGallery.
//
// iGallery is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// iGallery is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Foobar; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package org.ninjasoft.igallery.data;

import java.util.*;
import java.io.*;

/**
 * Class to load/save application properties.  It is mindful of the 
 * multiuser nature of the OS and sets permissions on the saved file
 * such that nobody else can read it (because it may contain a password).
 * 
 * @author enigma
 */
public class ApplicationProperties extends Properties {
    private String filename = ""; 
    public ApplicationProperties() throws IOException {
        super();
        filename = System.getProperty("user.home") + "/Library/Preferences/org.ninjasoft.igallery.properties";
        this.load();
    }
    
    public void load() throws IOException {
        File f = new File(filename);
        InputStream in = null;
        if (!f.exists())
            return;
        in = new FileInputStream(f);
        this.load(in);
        in.close();
    }
    
    public void save() throws IOException {
        File saveFile = new File(filename);
        File tempFile = new File(filename + ".temp");
        if (tempFile.exists())
            tempFile.delete();
        tempFile.createNewFile();
        // Boy, is this chmod cheesy:
        Runtime.getRuntime().exec("/bin/chmod 600 " + tempFile.getCanonicalPath());
        OutputStream out = new FileOutputStream(tempFile, true);
        this.store(out, "");
        out.close();
        saveFile.delete();
        tempFile.renameTo(saveFile);
        // The first chmod doesn't seem to stick...
        Runtime.getRuntime().exec("/bin/chmod 600 " + saveFile.getCanonicalPath());
    }
}
