// Copyright (C) 2004, Brian Enigma <enigma at netninja.com>
// This file is part of iGallery.
//
// iGallery is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// iGallery is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Foobar; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package org.ninjasoft.igallery.logic;

import java.io.*;
import java.util.*;

import org.ninjasoft.igallery.data.*;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;
import javax.xml.parsers.*;

/**
 * SAX handler to parse the AlbumData.xml file
 * @author enigma
 */
public class AlbumSaxHandler extends DefaultHandler {
    private static final int MODE_NONE = 0;
    private static final int MODE_CAPTURE_ALBUMS = 1;
    private static final int MODE_CAPTURE_PICTURES = 2;
    public int mode = MODE_NONE;
    private StringBuffer characterData = new StringBuffer();
    private boolean recordCharacterData = false;
    private boolean recordPhotoIds = false;
    private String currentTag = "";
    private String currentKey = "";
    private IPhotoAlbumData currentAlbum = new IPhotoAlbumData();
    private IPhotoPictureData currentPicture = null;
    private int throttleCount = 0;
    private Vector albums = new Vector();
    private Hashtable pictureHash = new Hashtable();
    
    private void throttle() {
        if (throttleCount++ > 60)
            System.exit(0);
    }
    
	public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
		super.startElement(uri, localName, qName, attributes);
        currentTag = qName;
        if (currentTag.equals("key"))
            recordCharacterData = true;
        if ((currentKey.length() > 0) && (qName.equals("string")))
            recordCharacterData = true;
        if (recordPhotoIds)
            recordCharacterData = true;
        if (mode == MODE_CAPTURE_ALBUMS) {
            if ((currentAlbum == null) && qName.equals("dict"))
                currentAlbum = new IPhotoAlbumData();
        }
	}
    public void endElement(String uri, String localName, String qName) throws SAXException {
        super.endElement(uri, localName, qName);
        recordCharacterData = false;
        String value = characterData.toString();
        characterData.delete(0, characterData.length());
        if (currentTag.equals("key")) {
            if ((mode == MODE_NONE) && value.equals("List of Albums")) {
                System.out.println("Album mode");
                mode = MODE_CAPTURE_ALBUMS;
            } if ((mode == MODE_CAPTURE_ALBUMS) && value.equals("Master Image List")) {
                System.out.println("Picture mode");
                mode = MODE_CAPTURE_PICTURES;
            }
        }
        if (mode == MODE_CAPTURE_ALBUMS) {
            if (currentAlbum != null) {
                // Check if it's a key
                if (currentTag.equals("key")) {
                    value = value.trim();
                    currentKey = value;
                }
                // Check if it's a value, then record it associated with the key, if 
                // it's a key we care about
                if (currentTag.equals("string") && (currentKey.length() > 0)) {
                    value = value.trim();
                    if (currentKey.equals("AlbumName"))
                        currentAlbum.setName(value);
                    else if (currentKey.equals("Album Type"))
                        currentAlbum.setType(value);
                    else if (currentKey.equals("KeyList"))
                        recordPhotoIds = true;
                    currentKey = "";
                }
                // If we're recording photograph key IDs, add them
                if (recordPhotoIds && (currentTag.equals("string"))) {
                    int id = 0;
                    try{
                        id = Integer.parseInt(value);
                    }catch(Throwable t){}
                    if (id > 0)
                        currentAlbum.addPicture(id);
                }
                // If we're at the end of the photograph key IDs, stop recording
                if (recordPhotoIds && qName.equals("array"))
                    recordPhotoIds = false;
                // If we're at the end of the album record, store it
                if ((currentAlbum != null) && (qName.equals("dict"))) {
                    System.out.println(currentAlbum);
                    albums.add(currentAlbum);
                    currentAlbum = null;
                    currentKey = "";
                }
            }
        }
        // In capture-picture mode
        if (mode == MODE_CAPTURE_PICTURES) {
            // If we aren't currently capturing a picture record, start one if this is the start of one
            if ((currentPicture == null) && currentTag.equals("key")) {
                int id = 0;
                try{
                    id = Integer.parseInt(value);
                }catch(Throwable t){}
                if (id > 0) {
                	currentPicture = new IPhotoPictureData();
                	currentPicture.setId(id);
                    currentKey = "";
                }
            }
            if ((currentPicture != null) && (currentTag.equals("key"))) {
                value = value.trim();
                currentKey = value;
            }
            if ((currentPicture != null) && currentTag.equals("string") && (currentKey.length() > 0)) {
                value = value.trim();
                if (currentKey.equals("Caption"))
                    currentPicture.setCaption(value);
                else if (currentKey.equals("ImagePath"))
                    currentPicture.setFilename(value);
                currentKey = "";
            }
            // If this is the end of the picture record, save it
            if ((currentPicture != null) && (qName.equals("dict"))) {
                //System.out.println(currentPicture);
                pictureHash.put(currentPicture.getId(), currentPicture);
                currentPicture = null;
                currentKey = "";
            }
        }
        currentTag = "";
    }
    public void characters(char[] data, int start, int length) throws SAXException {
        super.characters(data, start, length);
        if (recordCharacterData)
            characterData.append(data, start, length);
    }
    
    public Vector getAlbums() {
        return this.albums;
    }
    
    public Hashtable getPictureHash() {
        return this.pictureHash;
    }
    
    public static void main(String[] argv) throws Exception {
        AlbumSaxHandler handler = new AlbumSaxHandler();
        System.out.println("Scanning...");
        SAXParserFactory.newInstance().newSAXParser().parse(
                new FileInputStream("/Users/enigma/Desktop/AlbumDataCopy.xml"), 
                handler);
        System.out.println("Obtained data about " + handler.getAlbums().size() + " album(s), " + handler.getPictureHash().size() + " picture(s)");
    }
}
