// Copyright (C) 2004, Brian Enigma <enigma at netninja.com>
// This file is part of iGallery.
//
// iGallery is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// iGallery is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Foobar; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package org.ninjasoft.igallery.logic;

import java.util.*;
import java.net.*;

/**
 * Persist cookies across URLConnections
 * @author enigma
 */
public class Cookiejar {
    private StatusListener listener = null;
    public static boolean DEBUG = false;
    
    private Vector cookies = new Vector();
    
    /**
     * Given an incoming connection, grab any Set-Cookie: headers from the
     * connection and store them for later
     * @param connection
     */
    public void grabCookies(URLConnection connection) {
        // Headers are on a ONE-BASED index instead of zero-based
        int i = 1;
        String key;
        while ((key = connection.getHeaderFieldKey(i)) != null) {
            if (key.equalsIgnoreCase("set-cookie")) {
                String value = connection.getHeaderField(i);
                addCookie(value);
            }
            i++;
        }
    }
    
    /**
     * Given an outbound connection, set all cookies in that connection.
     * @param connection
     */
    public void setCookies(URLConnection connection ) {
        for (Iterator i = cookies.iterator(); i.hasNext(); ) {
            String value = (String) i.next();
            if (DEBUG)
            	sendStatus("Adding cookie " + value);
            connection.addRequestProperty("Cookie", value);
        }
    }
    
    
    private void addCookie(String cookieData) {
        // TODO: parse cookie data and replace/expire stuff
        int pos = cookieData.indexOf(';');
        if (pos > 0)
            cookieData = cookieData.substring(0, pos);
        pos = cookieData.indexOf('=');
        if (pos > 0) {
            String key = cookieData.substring(0, pos+1);
            Object[] cookieArray = cookies.toArray();
            for (int i=0; i<cookieArray.length; i++) {
                String testValue = (String) cookieArray[i];
                if (testValue.startsWith(key)) {
                    if (DEBUG)
                        sendStatus("Removing existing cookie " + testValue);
                    cookies.remove(testValue);
                }
            }
        }
        if (DEBUG)
            sendStatus("Adding cookie " + cookieData);
        cookies.add(cookieData);
    }
    
    private void cleanupCookies() {
        // TODO: expire cookies, etc 
    }
    
    public void setStatusListener(StatusListener l)     {this.listener = l;}

    private void sendStatus(String message) {
        if (listener != null)
            listener.setStatus(message);
    }
    private void sendError(String message) {
        if (listener != null)
            listener.setError(message);
    }
}
